# chown_r
# Defined Resource Type to carry out chown -R when required.  This avoids
# having to use recursive `file` resources with large directories as this
# can bring about poor performance in the Puppet Master as well as placing
# high demand on storage requirements
#
#
# Params
# ======
#   [`dir`]
#     The directory to check and chown.  Defaults to $name
#   [`want_user`]
#     The user ID to chown to and check for
#   [`want_group`]
#     the group ID to chown to and check for
#   [`watch`]
#     Resource reference to watch (eg Package['foo']), if set, we will only
#     run the chown if this resource sends a refresh event AND we identify
#     files with incorrect ownership.  Note that if this parameter is not set,
#     the chown command will run every time find tells us we need to (eg
#     every puppet run)
# @param skip Do not include this directory when running chmod

define chown_r(
  $want_user,
  $want_group,
  $dir   = $name,
  $watch = false,
  $skip  = false,
) {

  if $watch {
    $refreshonly  = true
    $_watch       = $watch
  } else {
    $refreshonly  = false
    $_watch       = undef
  }

  # must use '!' instead of -not on solaris or we get a bad command error
  if $facts['os']['family'] == 'Solaris' {
    $not_pred  = '!'
    $or_pred   = '-o'
  } else {
    $not_pred  = '-not'
    $or_pred   = '-or'
  }

  if $skip {
    $_skip = "-wholename ${skip} -prune -o"
  } else {
    $_skip = ""
  }

  if $want_user {
    $_want_user = "${not_pred} -user ${want_user}"
  } else {
    $_want_user = ""
  }

  if $want_group {
    $_want_group = "${not_pred} -group ${want_group}"
  } else {
    $_want_group = ""
  }

  if $want_user and $want_group {
    $want_both = $or_pred
    $cmd = "chown ${want_user}:${want_group}"
  } elsif $want_user {
    $want_both = ""
    $cmd = "chown ${want_user}"
  } elsif $want_group {
    $want_both = ""
    $cmd = "chgrp ${want_group}"
  } else {
    fail("One of want_user or want_group must be specified")
  }

  # change ownership if find matches any files or directories with different
  # ownership to $want_user or $want_group
  exec { "chown -R for ${dir}":
    command     => "find ${dir} ${_skip} -exec ${cmd} {} \\;",
    refreshonly => $refreshonly,
    onlyif      => "find ${dir} ${_skip} \\( ${_want_user} ${want_both} ${_want_group} \\) -print | grep .",
    subscribe   => $_watch,
    path        => [
      "/bin",
      "/usr/bin",
    ],
  }

}
